/*
________________________________________________________________________________

   01.lookup-codes.sas 
   OPA Measure Calculations
   Step 1: Read in claims code lookup tables

   Summary  : Downloads lookup table and saves each worksheet to a SAS dataset.
   Saves codes into macro variables. 

   Authors  : P. Hastings / phil -at- farharbor -dot- com
            : F. Dong / fei -at- farharbor -dot- com 
			: E. Puga / ella -at- farharbor -dot- com 
            : Based on original code by B. Frederiksen -at- DHHS/OPA
   Version  : v3.01
   Date     : 2020-03-20
   Revisions: Updated lookup table names to include _v3. 
   History  : 2016-12-19(1.00), 2017-02-22(1.01), 2017-05-17(1.02), 
              2018-04-20(1.03), 2018-08-20(1.03a), 2019-02-26 (2.01)

________________________________________________________________________________

*/

*...DO NOT EDIT: macro variables for setup;
%let DiagCodes = &DiagCodePrefix:;
%let DrugCodes = &DrugCodePrefix:;
%let ProcCodes = &ProcCodePrefix:;

*...Macro to standardize postpartum macro variable. Displays note in log if postpartum variable is set to no, blank, or has an invalid entrry that programs will default to all women (no
postpartum). If postpartum variable is set to yes, default calculations will be performed at 3 and 90 days postpartum unless otherwise specified in dayspp_list variable.;
%macro standpp/minoperator;
%symdel postpartum_st /nowarn ;
%global postpartum_st;
%if %length(&postpartum.) > 0 %then 
   %do;
   %if &postpartum. in yes YES Yes y Y %then 
      %do;
      %let postpartum_st = y;
      %end;
   %else %if &postpartum. in no NO No n N %then 
      %do;
      %let postpartum_st = n;
      %let dayspp_list = ;
      %put ;
      %put NOTE: Calculate contraceptive measures for all women.;
      %put NOTE: If interested in postpartum subpopulation, edit postpartum = yes in setup.;
      %put ;
      %end;
   %else 
      %do;
      %let postpartum_st = n;
      %let dayspp_list = ;
      %put ;
      %put NOTE: An entry other than yes or no was entered in the postpartum macro variable.;
      %put NOTE: Programs will default to all women. If interested in postpartum subpopulation, edit postpartum = yes in setup.; 
      %put ;
   %end;
%end;
%else 
   %do;
   %put ;
   %let postpartum_st = n;
   %let dayspp_list = ;
   %put NOTE: Calculate contraceptive measures for all women.;
   %put NOTE: If interested in postpartum subpopulation, edit postpartum = yes in setup.;
   %put ;
   %end;
%if &postpartum_st. = y %then 
   %do;
   %if %length(&dayspp_list.) = 0 %then 
      %do;
      %let dayspp_list = 3 90;
      %put ;
      %put NOTE: Default postpartum calculations will be performed at 3 and 90 days postpartum.;
      %put NOTE: If interested in other timeframes, edit dayspp_list = #days in setup.;
      %put ;
      %end;
   %else %do;
      %put ;
      %put NOTE: Postpartum calculations will be performed at %trim(&dayspp_list) days postpartum.;
      %put NOTE: If interested in other timeframes, edit dayspp_list = #days in setup.;
      %put ;
      %end;
   %end;
%mend standpp;
%standpp;
   
*===;
*...Save codes from OPA lookup tables into macro variables for later use;
*===;
proc sql noprint;
   select quote(strip(Diagnosis)) into :InfecundDiag separated by " " from lookup_Infecund_v3_&year. where missing(Diagnosis) ne 1;
   select quote(strip(Procedure)) into :InfecundProc separated by " " from lookup_Infecund_v3_&year. where missing(Procedure) ne 1;
   
   select quote(strip(Diagnosis)) into :PregDiag separated by " " from lookup_Pregnancy_v3_&year. where missing(Diagnosis) ne 1;
   select quote(strip(Procedure)) into :PregProc separated by " " from lookup_Pregnancy_v3_&year. where missing(Procedure) ne 1;
   
   select quote(strip(Diagnosis)) into :NonLiveBirthDiag separated by " " from lookup_Non_live_birth_v3_&year. where missing(Diagnosis) ne 1;
   select quote(strip(Procedure)) into :NonLiveBirthProc separated by " " from lookup_Non_live_birth_v3_&year. where missing(Procedure) ne 1;

   select quote(strip(Diagnosis)) into :LiveBirthDiag separated by " " from lookup_Live_birth_v3_&year. where missing(Diagnosis) ne 1;
   select quote(strip(Procedure)) into :LiveBirthProc separated by " " from lookup_Live_birth_v3_&year. where missing(Procedure) ne 1;

   select quote(strip(Diagnosis)) into :FemSterilDiag separated by " " from lookup_Sterilization_v3_&year. where missing(Diagnosis) ne 1;
   select quote(strip(Procedure)) into :FemSterilProc separated by " " from lookup_Sterilization_v3_&year. where missing(Procedure) ne 1;
   select quote(strip(NDC))       into :FemSterilDrug separated by " " from lookup_Sterilization_v3_&year. where missing(NDC) ne 1;

   select quote(strip(Diagnosis)) into :IUDDiag separated by " " from lookup_IUD_v3_&year. where missing(Diagnosis) ne 1;
   select quote(strip(Procedure)) into :IUDProc separated by " " from lookup_IUD_v3_&year. where missing(Procedure) ne 1;
   select quote(strip(NDC))       into :IUDDrug separated by " " from lookup_IUD_v3_&year. where missing(NDC) ne 1;

   select quote(strip(Diagnosis)) into :ImplantDiag separated by " " from lookup_Implant_v3_&year. where missing(Diagnosis) ne 1;
   select quote(strip(Procedure)) into :ImplantProc separated by " " from lookup_Implant_v3_&year. where missing(Procedure) ne 1;
   select quote(strip(NDC))       into :ImplantDrug separated by " " from lookup_Implant_v3_&year. where missing(NDC) ne 1;

   select quote(strip(Diagnosis)) into :InjectDiag separated by " " from lookup_Injectable_v3_&year. where missing(Diagnosis) ne 1; 
   select quote(strip(Procedure)) into :InjectProc separated by " " from lookup_Injectable_v3_&year. where missing(Procedure) ne 1;
   select quote(strip(NDC))       into :InjectDrug separated by " " from lookup_Injectable_v3_&year. where missing(NDC) ne 1;

   select quote(strip(Diagnosis)) into :OCDiag separated by " " from lookup_Oral_Pill_v3_&year. where missing(Diagnosis) ne 1;
   select quote(strip(Procedure)) into :OCProc separated by " " from lookup_Oral_Pill_v3_&year. where missing(Procedure) ne 1;
   select quote(strip(NDC))       into :OCDrug separated by " " from lookup_Oral_Pill_v3_&year. where missing(NDC) ne 1;

   select quote(strip(Diagnosis)) into :PatchDiag separated by " " from lookup_Patch_v3_&year. where missing(Diagnosis) ne 1; 
   select quote(strip(Procedure)) into :PatchProc separated by " " from lookup_Patch_v3_&year. where missing(Procedure) ne 1;
   select quote(strip(NDC))       into :PatchDrug separated by " " from lookup_Patch_v3_&year. where missing(NDC) ne 1;

   select quote(strip(Diagnosis)) into :VagRingDiag separated by " " from lookup_Ring_v3_&year. where missing(Diagnosis) ne 1; 
   select quote(strip(Procedure)) into :VagRingProc separated by " " from lookup_Ring_v3_&year. where missing(Procedure) ne 1;
   select quote(strip(NDC))       into :VagRingDrug separated by " " from lookup_Ring_v3_&year. where missing(NDC) ne 1;

   *...FD EDIT === UPDATE Feb 2019 === ;
   *...FD: use the LARC tab in the updated lookup table, which has no surveillance codes;   
   select quote(strip(Diagnosis)) into :LARCNosurvDiag separated by " " from lookup_larc_v3_&year. where missing(Diagnosis) ne 1;
   select quote(strip(Procedure)) into :LARCNosurvProc separated by " " from lookup_larc_v3_&year. where missing(Procedure) ne 1;
   select quote(strip(NDC))       into :LARCNosurvDrug separated by " " from lookup_larc_v3_&year. where missing(NDC) ne 1;

quit; 

*===;
*  Categorize most, moderately effective, and LARC-related methods;
*===;

*...Most effective methods (Female Sterilization, Intrauterine device (IUD/IUS), and Hormonal implant);
%let MostEffectProc=&FemSterilProc &IUDProc &ImplantProc;
%let MostEffectDiag=&FemSterilDiag &IUDDiag &ImplantDiag;
%let MostEffectDrug=&FemSterilDrug &IUDDrug &ImplantDrug; 

*...Moderately effective methods (Injectable, Oral contraceptive, Patch, Ring);
%let ModEffectProc=&InjectProc &OCProc &PatchProc &VagRingProc;
%let ModEffectDiag=&InjectDiag &OCDiag &PatchDiag &VagRingDiag;
%let ModEffectDrug=&InjectDrug &OCDrug &PatchDrug &VagRingDrug;

*...Most/moderately effective methods (Female Sterilization, Intrauterine device (IUD/IUS), 
    Hormonal implant, Injectable, Oral contracptive, Patch, Ring);
%let MostModProc=&FemSterilProc &IUDProc &ImplantProc &InjectProc &OCProc &PatchProc &VagRingProc;
%let MostModDiag=&FemSterilDiag &IUDDiag &ImplantDiag &InjectDiag &OCDiag &PatchDiag &VagRingDiag; 
%let MostModDrug=&FemSterilDrug &IUDDrug &ImplantDrug &InjectDrug &OCDrug &PatchDrug &VagRingDrug; 

*...FD EDIT === UPDATE Feb 2019 === ;
*...FD: NEW LARC codes = IUD/IUS + Hormonal Implants, without surveillance;
%let LARCProc=&LARCNosurvProc ;
%let LARCDiag=&LARCNosurvDiag ;
%let LARCDrug=&LARCNosurvDrug ;



